@@ -1,5 +1,5 @@
 /* `time' utility to display resource usage of processes.
-   Copyright (C) 1989, 1991, 1992 Free Software Foundation, Inc.
+   Copyright (C) 1990, 1991, 1992, 1993 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -15,119 +15,48 @@
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
-/* Most information is collected from the rusage struct.
-
-   Usage: time [-avp] [-f format] [-o file] [--append] [--portability]
-          [--verbose] [--format format] [--output-file file] command [arg...]
-
-   Options:
-   -a, --append			Append to output instead of overwriting.
-   -f, --format STRING		Use STRING to format the statistics.
-   -o, --output-file NAME	Send summary to NAME instead of stderr.
-   -p, --portability		Use the POSIX.2 output format.
-   -v, --verbose		Use a verbose format to print the statistics.
-
-   Written by David Keppel, pardo@cs.washington.edu.
-   Modified by David MacKenzie, djm@gnu.ai.mit.edu. */
+/* Originally written by David Keppel, pardo@cs.washington.edu.
+   Heavily modified by David MacKenzie, djm@gnu.ai.mit.edu.  */
 
+static const char *version_string = "GNU time version 1.5";
+
 #include <stdio.h>
 #include <sys/types.h>
+#include <sys/param.h>		/* For getpagesize, maybe.  */
+#ifndef SIGINT			/* Thank you, OSF/1.  */
 #include <signal.h>
+#endif
 #include <errno.h>
-#include <getopt.h>
-#ifdef linux
-#include <sys/wait.h>
-#endif
-
-#ifdef HAVE_SYS_TIME_H
-#include <sys/time.h>
-#else
-#include "timeval.h"
-#endif
-
-#ifdef HAVE_SYS_RESOURCE_H
-#include <sys/resource.h>
-#include <sys/wait.h>
-#else
-#include "rusage.h"
-#endif
-
-#if defined(__STDC__)
-# define P_(s) s
-# define PTR void *
-#else
-# define P_(s) ()
-# define PTR char *
-# define const
-#endif
-
-#if defined(STDC_HEADERS) || defined(USG)
-#include <string.h>
-#else
-#include <strings.h>
-#endif
-
-#ifdef STDC_HEADERS
-#include <limits.h>
-#include <stdlib.h>
-#else
-#define	LONG_MAX (~(1 << (sizeof (long) * 8 - 1)))
-void abort P_((void));
-void exit P_((int));
-char *getenv P_((const char *));
-PTR malloc P_((size_t));
+#ifndef errno
 extern int errno;
 #endif
-
-#ifdef HAVE_UNISTD_H
-#include <unistd.h>
-#endif
-
-#ifndef _POSIX_VERSION
-int execvp P_((const char *, const char **));
-int fork P_((void));
-void _exit P_((int));
-#endif /* !_POSIX_VERSION */
-
-void error P_((int status, int errnum, char *message, ...));
-
-int gettimeofday P_((struct timeval *tp, struct timezone *tz));
-#ifdef HAVE_SYS_RESOURCE_H
-int wait3 P_((union wait *status, int options, struct rusage *rusage));
-#else
-int wait3 P_((int *status, int options, struct rusage *rusage));
-#endif
-
+#include <getopt.h>
+#include "port.h"
+#include "resuse.h"
 #include "getpagesize.h"
-#if !defined(getpagesize) && !defined(linux)
-int getpagesize P_((void));
-#endif
 
-void usage P_((void));
+void error PARAMS((int status, int errnum, char *message, ...));
+
+static void usage PARAMS((FILE *, int));
 
-#ifndef RETSIGTYPE
-#define RETSIGTYPE void
-#endif
+/* A Pointer to a signal handler.  */
 typedef RETSIGTYPE (*sighand_t) ();
 
-/* The time in msec of one `tick' as used by the `getrusage' structure.
+/* The number of milliseconds in one `tick' used by the `rusage' structure.
+
    At least one Unix man page has an error, as it reports
-   units for average sizes in kb*sec.  Judging by the
-   output of `time' (all versions), this looks like it
-   ought to be in kb*ticks. */
-#define ONETICK (20)		/* 20 msec/tick, 1/50th sec. */
+   units for average sizes as kb*sec.  Judging by the output
+   of `time' (all versions), this looks like it should be kb*ticks.
 
-/* Convert T msec to a number of ticks. */
-#define TICK(t) ((t) / ONETICK)
+   1 tick = 20 msec = 1/50 sec.  */
+#define ONETICK (20)
 
-typedef enum
-{
-  NO = 0, YES
-} bool;
+/* Convert m milliseconds to ticks.  */
+#define MSEC_TO_TICKS(m) ((m) / ONETICK)
+
+typedef enum {false, true} boolean;
 
 #define UL unsigned long
-
-char *version_string = "GNU time version 1.4";
 
 /* The default output format.  */
 static const char *const default_format =
@@ -138,29 +67,28 @@ static const char *const default_format =
 static const char *const posix_format = "real %e\nuser %U\nsys %S";
 
 /* Format string for printing all statistics verbosely.
+   Keep this output to 24 lines so users on terminals can see it all.
+
    The format string is used two ways: as a format string, and in
    verbose mode, to document all the possible formatting possiblities.
    When `longstats' is used as a format string, it has to be put into
    one contiguous string (e.g., into a `char[]').  We could alternatively
    store it as a `char *' and convert it into a `*char[]' when we need
-   it as documentation, but most compilers choke on very long strings. */
+   it as documentation, but most compilers choke on very long strings.  */
+
 static const char *const longstats[] =
 {
-  "\nSummary statistics:\n",
   "\tCommand being timed: \"%C\"\n",
-  "\n",
   "\tUser time (seconds): %U\n",
   "\tSystem time (seconds): %S\n",
   "\tPercent of CPU this job got: %P\n",
   "\tElapsed (wall clock) time (h:mm:ss or m:ss): %E\n",
-  "\n",
   "\tAverage shared text size (kbytes): %X\n",
   "\tAverage unshared data size (kbytes): %D\n",
   "\tAverage stack size (kbytes): %p\n",
   "\tAverage total size (kbytes): %K\n",
   "\tMaximum resident set size (kbytes): %M\n",
   "\tAverage resident set size (kbytes): %t\n",
-  "\n",
   "\tMajor (requiring I/O) page faults: %F\n",
   "\tMinor (reclaiming a frame) page faults: %R\n",
   "\tVoluntary context switches: %w\n",
@@ -168,57 +96,53 @@ static const char *const longstats[] =
   "\tSwaps: %W\n",
   "\tFile system inputs: %I\n",
   "\tFile system outputs: %O\n",
-  "\n",
   "\tSocket messages sent: %s\n",
   "\tSocket messages received: %r\n",
   "\tSignals delivered: %k\n",
-  "\n",
   "\tPage size (bytes): %Z\n",
-  "\tExit status: %x\n",
+  "\tExit status: %x",
   NULL
 };
 
-/* If YES, show an English description next to each statistic. */
-static bool verbose;
+/* If true, show an English description next to each statistic.  */
+static boolean verbose;
 
-/* Name of output file.  Only used if -o option is given. */
-static char const *outfile;
+/* Name of output file.  Only used if -o option is given.  */
+static const char *outfile;
 
-/* Output stream, stderr by default. */
+/* Output stream, stderr by default.  */
 static FILE *outfp;
 
-/* If YES, append to `outfile' rather than truncating it. */
-static bool append;
+/* If true, append to `outfile' rather than truncating it.  */
+static boolean append;
 
-/* Controls the output format. */
-static char const *output_format;
+/* The output format string.  */
+static const char *output_format;
 
-/* The command line to run and gather statistics on. */
-static char const **command_line;
-
-/* The name this program was invoked by. */
+/* The name this program was invoked by.  */
 char *program_name;
 
 static struct option longopts[] =
 {
-  {"append", 0, NULL, 'a'},
-  {"format", 1, NULL, 'f'},
-  {"output-file", 1, NULL, 'o'},
-  {"portability", 0, NULL, 'p'},
-  {"verbose", 0, NULL, 'v'},
-  {"version", 0, NULL, 'V'},
-  {NULL, 0, NULL, 0}
+  {"append", no_argument, NULL, 'a'},
+  {"format", required_argument, NULL, 'f'},
+  {"help", no_argument, NULL, 'h'},
+  {"output-file", required_argument, NULL, 'o'},
+  {"portability", no_argument, NULL, 'p'},
+  {"verbose", no_argument, NULL, 'v'},
+  {"version", no_argument, NULL, 'V'},
+  {NULL, no_argument, NULL, 0}
 };
 
-/* Print ARGV to FP, with each entry in ARGV separated by FILLER. */
+/* Print ARGV to FP, with each entry in ARGV separated by FILLER.  */
 
-void
+static void
 fprintargv (fp, argv, filler)
      FILE *fp;
-     char const *const argv[];
-     char const *filler;
+     const char *const *argv;
+     const char *filler;
 {
-  char const *const *av;
+  const char *const *av;
 
   av = argv;
   fputs (*av, fp);
@@ -231,25 +155,69 @@ fprintargv (fp, argv, filler)
     error (1, errno, "write error");
 }
 
+/* Return a null-terminated string containing the concatenation,
+   in order, of all of the elements of ARGV.
+   The '\0' at the end of each ARGV-element is not copied.
+   Example:	char *argv[] = {"12", "ab", ".,"};
+ 		linear_argv(argv) == "12ab.,"
+   Print a message and return NULL if memory allocation failed.  */
+
+static char *
+linear_argv (argv)
+     const char *const *argv;
+{
+  const char *const *s;		/* Each string in ARGV.  */
+  char *new;			/* Allocated space.  */
+  char *dp;			/* Copy in to destination.  */
+  const char *sp;		/* Copy from source.  */
+  int size;
+
+  /* Find length of ARGV and allocate.  */
+  size = 1;
+  for (s = argv; *s; ++s)
+    size += strlen (*s);
+  new = (char *) malloc (size);
+  if (new == NULL)
+    {
+      fprintf (stderr, "%s: virtual memory exhausted\n", program_name);
+      return NULL;
+    }
+
+  /* Copy each string in ARGV to the new string.  At the end of
+     each string copy, back up `dp' so that on the next string,
+     the `\0' will be overwritten.  */
+  for (s = argv, sp = *s, dp = new; *s; ++s)
+    {
+      sp = *s;
+      while ((*dp++ = *sp++) != '\0')
+	/* Do nothing.  */ ;
+      --dp;
+    }
+
+  return new;
+}
+
 /* Return the number of kilobytes corresponding to a number of pages PAGES.
+   (Actually, we use it to convert pages*ticks into kilobytes*ticks.)
+
    Try to do arithmetic so that the risk of overflow errors is minimized.
    This is funky since the pagesize could be less than 1K.
    Note: Some machines express getrusage statistics in terms of K,
-   others in terms of pages. */
+   others in terms of pages.  */
 
-unsigned long
-tok (pages)
+static unsigned long
+ptok (pages)
      unsigned long pages;
 {
   static unsigned long ps = 0;
   unsigned long tmp;
   static long size = LONG_MAX;
 
-  /* Initialization. */
+  /* Initialization.  */
   if (ps == 0)
     ps = (long) getpagesize ();
 
-  /* Conversion. */
+  /* Conversion.  */
   if (pages > (LONG_MAX / ps))
     {				/* Could overflow.  */
       tmp = pages / 1024;	/* Smaller first, */
@@ -272,7 +240,7 @@ tok (pages)
    The character following a `%' can be:
    (* means the tcsh time builtin also recognizes it)
    % == a literal `%'
-   C == command name
+   C == command name and arguments
 *  D == average unshared data size in K (ru_idrss+ru_isrss)
 *  E == elapsed real (wall clock) time in [hour:]min:sec.usec
 *  F == major page faults (required physical I/O) (ru_majflt)
@@ -288,6 +256,7 @@ tok (pages)
 *  X == average amount of shared text in K (ru_ixrss)
    Z == page size
 *  c == involuntary context switches (ru_nivcsw)
+   e == elapsed real time in sec.usec
 *  k == signals delivered (ru_nsignals)
    p == average unshared stack size in K (ru_isrss)
 *  r == socket messages received (ru_msgrcv)
@@ -303,34 +272,30 @@ tok (pages)
    FP is the stream to print to.
    FMT is the format string, interpreted as described above.
    COMMAND is the command and args that are being summarized.
-   RUP is information on the command.
-   TV is the elapsed time of the command.
-   EXITSTAT is the command's exit status. */
+   RESP is information on the command.  */
 
-void
-summarize (fp, fmt, command, rup, tv, exitstat)
+static void
+summarize (fp, fmt, command, resp)
      FILE *fp;
-     char const *fmt;
-     char const *command[];
-     struct rusage *rup;
-     struct timeval *tv;
-     int exitstat;
+     const char *fmt;
+     const char **command;
+     RESUSE *resp;
 {
-  unsigned long r;		/* Elapsed real milliseconds. */
-  unsigned long v;		/* Elapsed virtual (CPU) time. */
+  unsigned long r;		/* Elapsed real milliseconds.  */
+  unsigned long v;		/* Elapsed virtual (CPU) milliseconds.  */
 
-  if (exitstat)
-    fprintf (fp, "Command had non-zero exit status %d\n", exitstat);
+  if (resp->exitstatus)
+    fprintf (fp, "Command had non-zero exit status %d\n", resp->exitstatus);
 
-  /* All times converted to milliseconds.  `r' is real, or wallclock
-     time.  `v' is virtual, or cpu-seconds time.  Occasionally, one
-     of these values comes out as zero.  Dividing by zero causes
-     problems, so we first need to check the time value.  If it is
-     zero, then we take `evasive action' instead of printing a
-     value. */
-  r = tv->tv_sec * 1000 + tv->tv_usec / 1000;
-  v = rup->ru_utime.tv_sec * 1000 + rup->ru_utime.tv_usec / 1000 +
-    rup->ru_stime.tv_sec * 1000 + rup->ru_stime.tv_usec / 1000;
+  /* Convert all times to milliseconds.  Occasionally, one of these values
+     comes out as zero.  Dividing by zero causes problems, so we first
+     check the time value.  If it is zero, then we take `evasive action'
+     instead of calculating a value.  */
+
+  r = resp->elapsed.tv_sec * 1000 + resp->elapsed.tv_usec / 1000;
+
+  v = resp->ru.ru_utime.tv_sec * 1000 + resp->ru.ru_utime.TV_MSEC +
+    resp->ru.ru_stime.tv_sec * 1000 + resp->ru.ru_stime.TV_MSEC;
 
   while (*fmt)
     {
@@ -343,42 +308,44 @@ summarize (fp, fmt, command, rup, tv, exitstat)
 	      putc ('%', fp);
 	      break;
 	    case 'C':		/* The command that got timed.  */
-	      /* If no command, print nothing.  */
-	      if (*command)
-		fprintargv (fp, command, " ");
+	      fprintargv (fp, command, " ");
 	      break;
 	    case 'D':		/* Total unshared data size.  */
-	      fprintf (fp, "%lu", (TICK (v) == 0)
-		       ? 0
-		       : tok ((UL) rup->ru_idrss) / TICK (v) +
-		       tok ((UL) rup->ru_isrss) / TICK (v));
+	      fprintf (fp, "%lu",
+		       MSEC_TO_TICKS (v) == 0 ? 0 :
+		       ptok ((UL) resp->ru.ru_idrss) / MSEC_TO_TICKS (v) +
+		       ptok ((UL) resp->ru.ru_isrss) / MSEC_TO_TICKS (v));
 	      break;
 	    case 'E':		/* Elapsed real (wall clock) time.  */
-	      if (tv->tv_sec >= 3600)
-		fprintf (fp, "%ld:%02ld:%02ld", tv->tv_sec / 3600,
-			 (tv->tv_sec % 3600) / 60, tv->tv_sec % 60);
+	      if (resp->elapsed.tv_sec >= 3600)	/* One hour.  */
+		fprintf (fp, "%ld:%02ld:%02ld",
+			 resp->elapsed.tv_sec / 3600,
+			 (resp->elapsed.tv_sec % 3600) / 60,
+			 resp->elapsed.tv_sec % 60);
 	      else
-		fprintf (fp, "%ld:%02ld.%02ld", tv->tv_sec / 60,
-			 tv->tv_sec % 60, tv->tv_usec / 10000);
+		fprintf (fp, "%ld:%02ld.%02ld",
+			 resp->elapsed.tv_sec / 60,
+			 resp->elapsed.tv_sec % 60,
+			 resp->elapsed.tv_usec / 10000);
 	      break;
 	    case 'F':		/* Major page faults.  */
-	      fprintf (fp, "%ld", rup->ru_majflt);
+	      fprintf (fp, "%ld", resp->ru.ru_majflt);
 	      break;
 	    case 'I':		/* Inputs.  */
-	      fprintf (fp, "%ld", rup->ru_inblock);
+	      fprintf (fp, "%ld", resp->ru.ru_inblock);
 	      break;
 	    case 'K':		/* Average mem usage == data+stack+text.  */
 	      fprintf (fp, "%lu",
-		       (TICK (v) == 0) ? 0 :
-		       tok ((UL) rup->ru_idrss) / TICK (v) +
-		       tok ((UL) rup->ru_isrss) / TICK (v) +
-		       tok ((UL) rup->ru_ixrss) / TICK (v));
+		       MSEC_TO_TICKS (v) == 0 ? 0 :
+		       ptok ((UL) resp->ru.ru_idrss) / MSEC_TO_TICKS (v) +
+		       ptok ((UL) resp->ru.ru_isrss) / MSEC_TO_TICKS (v) +
+		       ptok ((UL) resp->ru.ru_ixrss) / MSEC_TO_TICKS (v));
 	      break;
 	    case 'M':		/* Maximum resident set size.  */
-	      fprintf (fp, "%lu", tok ((UL) rup->ru_maxrss));
+	      fprintf (fp, "%lu", ptok ((UL) resp->ru.ru_maxrss));
 	      break;
 	    case 'O':		/* Outputs.  */
-	      fprintf (fp, "%ld", rup->ru_oublock);
+	      fprintf (fp, "%ld", resp->ru.ru_oublock);
 	      break;
 	    case 'P':		/* Percent of CPU this job got.  */
 	      /* % cpu is (total cpu time)/(elapsed time).  */
@@ -388,54 +355,61 @@ summarize (fp, fmt, command, rup, tv, exitstat)
 		fprintf (fp, "?%%");
 	      break;
 	    case 'R':		/* Minor page faults (reclaims).  */
-	      fprintf (fp, "%ld", rup->ru_minflt);
+	      fprintf (fp, "%ld", resp->ru.ru_minflt);
 	      break;
 	    case 'S':		/* System time.  */
 	      fprintf (fp, "%ld.%02ld",
-		       rup->ru_stime.tv_sec, rup->ru_stime.tv_usec / 10000);
+		       resp->ru.ru_stime.tv_sec,
+		       resp->ru.ru_stime.TV_MSEC / 10);
 	      break;
 	    case 'U':		/* User time.  */
 	      fprintf (fp, "%ld.%02ld",
-		       rup->ru_utime.tv_sec, rup->ru_utime.tv_usec / 10000);
+		       resp->ru.ru_utime.tv_sec,
+		       resp->ru.ru_utime.TV_MSEC / 10);
 	      break;
 	    case 'W':		/* Times swapped out.  */
-	      fprintf (fp, "%ld", rup->ru_nswap);
+	      fprintf (fp, "%ld", resp->ru.ru_nswap);
 	      break;
 	    case 'X':		/* Average shared text size.  */
-	      fprintf (fp, "%lu", (TICK (v) == 0) ? 0
-		       : tok ((UL) rup->ru_ixrss) / TICK (v));
+	      fprintf (fp, "%lu",
+		       MSEC_TO_TICKS (v) == 0 ? 0 :
+		       ptok ((UL) resp->ru.ru_ixrss) / MSEC_TO_TICKS (v));
 	      break;
 	    case 'Z':		/* Page size.  */
 	      fprintf (fp, "%d", getpagesize ());
 	      break;
 	    case 'c':		/* Involuntary context switches.  */
-	      fprintf (fp, "%ld", rup->ru_nivcsw);
+	      fprintf (fp, "%ld", resp->ru.ru_nivcsw);
 	      break;
 	    case 'e':		/* Elapsed real time in seconds.  */
-	      fprintf (fp, "%ld.%02ld", tv->tv_sec, tv->tv_usec / 10000);
+	      fprintf (fp, "%ld.%02ld",
+		       resp->elapsed.tv_sec,
+		       resp->elapsed.tv_usec / 10000);
 	      break;
 	    case 'k':		/* Signals delivered.  */
-	      fprintf (fp, "%ld", rup->ru_nsignals);
+	      fprintf (fp, "%ld", resp->ru.ru_nsignals);
 	      break;
 	    case 'p':		/* Stack segment.  */
-	      fprintf (fp, "%lu", (TICK (v) == 0) ? 0
-		       : tok ((UL) rup->ru_isrss) / TICK (v));
+	      fprintf (fp, "%lu",
+		       MSEC_TO_TICKS (v) == 0 ? 0 :
+		       ptok ((UL) resp->ru.ru_isrss) / MSEC_TO_TICKS (v));
 	      break;
 	    case 'r':		/* Incoming socket messages received.  */
-	      fprintf (fp, "%ld", rup->ru_msgrcv);
+	      fprintf (fp, "%ld", resp->ru.ru_msgrcv);
 	      break;
 	    case 's':		/* Outgoing socket messages sent.  */
-	      fprintf (fp, "%ld", rup->ru_msgsnd);
+	      fprintf (fp, "%ld", resp->ru.ru_msgsnd);
 	      break;
 	    case 't':		/* Average resident set size.  */
 	      fprintf (fp, "%lu",
-		 (TICK (v) == 0) ? 0 : tok ((UL) rup->ru_idrss) / TICK (v));
+		       MSEC_TO_TICKS (v) == 0 ? 0 :
+		       ptok ((UL) resp->ru.ru_idrss) / MSEC_TO_TICKS (v));
 	      break;
 	    case 'w':		/* Voluntary context switches.  */
-	      fprintf (fp, "%ld", rup->ru_nvcsw);
+	      fprintf (fp, "%ld", resp->ru.ru_nvcsw);
 	      break;
 	    case 'x':		/* Exit status.  */
-	      fprintf (fp, "%d", exitstat);
+	      fprintf (fp, "%d", resp->exitstatus);
 	      break;
 	    case '\0':
 	      putc ('?', fp);
@@ -447,7 +421,7 @@ summarize (fp, fmt, command, rup, tv, exitstat)
 	  ++fmt;
 	  break;
 
-	case '\\':		/* Format escape. */
+	case '\\':		/* Format escape.  */
 	  switch (*++fmt)
 	    {
 	    case 't':
@@ -479,90 +453,52 @@ summarize (fp, fmt, command, rup, tv, exitstat)
   if (ferror (fp))
     error (1, errno, "write error");
 }
-
-/* Return a null-terminated string containing the concatenation,
-   in order, of all of the elements of ARGV.
-   The '\0' at the end of each ARGV-element is not copied.
-   Example:	char *argv[] = {"12", "ab", ".,"};
- 		linear_argv(argv) == "12ab.,"
-   Print a message and return NULL if memory allocation failed. */
-
-static char *
-linear_argv (argv)
-     char const *const *argv;
-{
-  char const *const *s;		/* Each string in ARGV. */
-  char *new;			/* Allocated space. */
-  char *dp;			/* Copy in to destination. */
-  char const *sp;		/* Copy from source. */
-  int size;
-
-  /* Find length of ARGV and allocate. */
-  size = 1;
-  for (s = argv; *s; ++s)
-    size += strlen (*s);
-  new = (char *) malloc (size);
-  if (new == NULL)
-    {
-      fprintf (stderr, "%s: virtual memory exhausted\n", program_name);
-      return NULL;
-    }
-
-  /* Copy each string in ARGV to the new string.  At the end of
-     each string copy, back up `dp' so that on the next string,
-     the `\0' will be overwritten. */
-  for (s = argv, sp = *s, dp = new; *s; ++s)
-    {
-      sp = *s;
-      while ((*dp++ = *sp++) != '\0')
-	/* Do nothing. */ ;
-      --dp;
-    }
-
-  return new;
-}
-
+
 /* Initialize the options and parse the command line arguments.
    Also note the position in ARGV where the command to time starts.
 
    By default, output is to stderr.
 
    ARGV is the array of command line arguments.
-   ARGC is the number of command line arguments. */
+   ARGC is the number of command line arguments.
 
-void
+   Return the command line to run and gather statistics on.  */
+
+static const char **
 getargs (argc, argv)
      int argc;
-     char *argv[];
+     char **argv;
 {
-  int optc, longind;
-  char *format;			/* Format found in environment. */
+  int optc;
+  char *format;			/* Format found in environment.  */
 
-  /* Initialize the option flags. */
-  verbose = NO;
+  /* Initialize the option flags.  */
+  verbose = false;
   outfile = NULL;
   outfp = stderr;
-  append = NO;
+  append = false;
   output_format = default_format;
   program_name = argv[0];
 
   /* Set the format string from the environment.  Do this before checking
-     the args so that we won't clobber a user-specified format. */
+     the args so that we won't clobber a user-specified format.  */
   format = getenv ("TIME");
   if (format)
     output_format = format;
 
-  while ((optc = getopt_long (argc, argv, "+af:o:pvV", longopts, &longind))
+  while ((optc = getopt_long (argc, argv, "+af:o:pvV", longopts, (int *) 0))
 	 != EOF)
     {
       switch (optc)
 	{
 	case 'a':
-	  append = YES;
+	  append = true;
 	  break;
 	case 'f':
 	  output_format = optarg;
 	  break;
+	case 'h':
+	  usage (stdout, 0);
 	case 'o':
 	  outfile = optarg;
 	  break;
@@ -570,20 +506,18 @@ getargs (argc, argv)
 	  output_format = posix_format;
 	  break;
 	case 'v':
-	  verbose = YES;
+	  verbose = true;
 	  break;
 	case 'V':
 	  fprintf (stderr, "%s\n", version_string);
-	  break;
+	  exit (0);
 	default:
-	  usage ();
+	  usage (stderr, 1);
 	}
     }
 
   if (optind == argc)
-    usage ();
-
-  command_line = (char const **) &argv[optind];
+    usage (stderr, 1);
 
   if (outfile)
     {
@@ -596,97 +530,50 @@ getargs (argc, argv)
     }
 
   /* If the user specified verbose output, we need to convert
-     `longstats' to a `char *'. */
+     `longstats' to a `char *'.  */
   if (verbose)
     {
-      output_format = (char const *) linear_argv (longstats);
+      output_format = (const char *) linear_argv (longstats);
       if (output_format == NULL)
-	exit (1);		/* Out of memory. */
+	exit (1);		/* Out of memory.  */
     }
+
+  return (const char **) &argv[optind];
 }
-
+
 /* Run command CMD and return statistics on it.
-   *RUSE gets most of the statistics.
-   *FINISH gets the elapsed time.
-   *EXITCODE gets the exit code of the child. */
+   Put the statistics in *RESP.  */
 
-void
-run_command (ruse, finish, exitcode, cmd)
-     struct rusage *ruse;
-     struct timeval *finish;
-     int *exitcode;
-#ifndef linux
-     char const *const cmd[];
-#else
-     char *const cmd[];
-#endif
+static void
+run_command (cmd, resp)
+     const char **cmd;
+     RESUSE *resp;
 {
-#ifdef HAVE_SYS_RESOURCE_H
-  union wait status;		/* Exit status of child. */
-#else
-  int status;
-#endif
-  int pid;			/* Pid of child. */
-  int caught;			/* Pid got back from wait. */
-  struct timeval start;		/* When job starts. */
+  pid_t pid;			/* Pid of child.  */
   sighand_t interrupt_signal, quit_signal;
 
-  /* Set time the command started.  Since this is BEFORE the
-     fork/exec sequence, this number is probably too large. */
-  gettimeofday (&start, (struct timezone *) NULL);
+  resuse_start (resp);
 
-  pid = fork ();		/* Run CMD as child process. */
+  pid = fork ();		/* Run CMD as child process.  */
   if (pid < 0)
     error (1, errno, "cannot fork");
   else if (pid == 0)
-    {				/* If child. */
-#ifndef linux
-      execvp (cmd[0], (const char **) cmd);
-#else
-      execvp (cmd[0], cmd);
-#endif
+    {				/* If child.  */
+      execvp (cmd[0], (char *const *) cmd);
       error (0, errno, "cannot run %s", cmd[0]);
       _exit (errno == ENOENT ? 127 : 126);
     }
 
-  /* Have signals kill the child but not self (if possible). */
+  /* Have signals kill the child but not self (if possible).  */
   interrupt_signal = signal (SIGINT, SIG_IGN);
   quit_signal = signal (SIGQUIT, SIG_IGN);
 
-  /* Ignore signals, but don't ignore the children.  When wait3
-     returns the child process, set the time the command finished. */
-  while ((caught = wait3 (&status, 0, ruse)) != pid)
-    {
-      if (caught == -1)
-	error (1, errno, "error waiting for child process");
-    }
+  if (resuse_end (pid, resp) == 0)
+    error (1, errno, "error waiting for child process");
 
-  /* Re-enable signals. */
+  /* Re-enable signals.  */
   signal (SIGINT, interrupt_signal);
   signal (SIGQUIT, quit_signal);
-
-  gettimeofday (finish, (struct timezone *) NULL);
-
-  /* Set `finish' to hold the elapsed "real" (wallclock) time of
-     the command.  If finish.usec < start.usec, manually carry a one
-     from the seconds field. */
-  finish->tv_sec -= start.tv_sec;
-  if (finish->tv_usec < start.tv_usec)
-    {
-      finish->tv_usec += 1000000;
-      --finish->tv_sec;
-    }
-  finish->tv_usec -= start.tv_usec;
-
-#ifdef HAVE_SYS_RESOURCE_H
-#ifndef linux
-  *exitcode = (int) status.w_T.w_Retcode;
-#else
-  *exitcode = (int) status.w_retcode;
-#endif
-#else
-  *exitcode = status >> 8;
-#endif
 }
 
 void
@@ -694,23 +581,24 @@ main (argc, argv)
      int argc;
      char **argv;
 {
-  int exitcode;			/* Exit status of job we ran. */
-  struct timeval elapsed;	/* Wallclock time of job. */
-  struct rusage ruse;		/* Use by children. */
+  const char **command_line;
+  RESUSE res;
 
-  getargs (argc, argv);
-  run_command (&ruse, &elapsed, &exitcode, command_line);
-  summarize (outfp, output_format, command_line, &ruse, &elapsed, exitcode);
-  exit (exitcode);		/* Exit with child's status. */
+  command_line = getargs (argc, argv);
+  run_command (command_line, &res);
+  summarize (outfp, output_format, command_line, &res);
+  exit (res.exitstatus);	/* Exit with child's status.  */
 }
 
-void
-usage ()
+static void
+usage (stream, status)
+     FILE *stream;
+     int status;
 {
-  fprintf (stderr, "\
-Usage: %s [-apvV] [-f format] [-o file] [--append] [--portability]\n\
-       [--verbose] [--format=format] [--output-file=file] [--version]\n\
-       command [arg...]\n",
+  fprintf (stream, "\
+Usage: %s [-apvV] [-f format] [-o file] [--append] [--verbose]\n\
+       [--portability] [--format=format] [--output=file] [--version]\n\
+       [--help] command [arg...]\n",
 	   program_name);
-  exit (1);
+  exit (status);
 }
